require 'claide'
require 'json'
require "time"
module GitThin

    class Expire < Thin
        include GitThinUtils
        self.summary = 'Delete expire branch'
        # self.plugin_prefixes = %w(claide cocoapods)
        self.arguments = [
          CLAide::Argument.new('BRANCHS', false ),
        ]

        self.description = <<-DESC
    Deletes a list of branches. Branches are specified by [BRANCHS] parameter, which can be a list of branch names or a file
        DESC

        def self.options
            [
              ['--source_root', 'Specify the warehouse address manually if necessary.'],
              ['--exclude_branchs', 'The name of the excluded branch，file or string.'],
            ].concat(super)

        end
        def initialize(argv)
            super

            @branchs = argv.shift_argument
            if not @branchs
                return
            end
            @source_root = argv.option('source_root')
            @exclude_branchs = argv.option('exclude_branchs')
            if @source_root
                Dir.chdir @source_root
            end
            run_shell 'git rev-parse --show-toplevel',false ,LOGNone do |out,err,status|
                if status == 0
                    @source_root = out[0].strip
                end
            end

            if not Dir.exist? @source_root
                @source_root = nil
                return
            end

            if FileTest.exist? @branchs
                File.open(@branchs, "r") do |aFile|
                    @branchs = ""
                    aFile.each do |line|
                        @branchs += line
                    end
                end
            end
            @branchs = @branchs.gsub(',',' ')
            @branchs = @branchs.split ' '

            if FileTest.exist? @exclude_branchs
                File.open(@exclude_branchs, "r") do |aFile|
                    @exclude_branchs = ""
                    aFile.each do |line|
                        @exclude_branchs += line
                    end
                end
            end
            @exclude_branchs = @exclude_branchs.gsub(',',' ')
            @exclude_branchs = @exclude_branchs.split ' '

            run_shell 'git branch -r',false ,true do |out,err,status|
                if status == 0
                    @remote_branchs = out.map { |line| line = line.strip }
                    @remote_branchs.delete_if do |item|
                        ret = false
                        if item.include? '->'
                            ret = true
                        end
                        ret
                    end
                end
            end

        end

        def validate!
            super
            help! 'validate SOURCE_ROOT is required.' unless @source_root
        end

        def expire_branchs
            Dir.chdir(@source_root)
            error_branchs = []
            exclude_count = 0
            @branchs.each do |branch|
                if @exclude_branchs.include? branch
                    logN "#{branch}分支名字黑名单，跳过"
                    exclude_count += 1
                else
                    if @remote_branchs.include? branch
                        index = branch.index('/')
                        branch[index] = ' '
                        # "git push -d #{branch}"
                        run_shell "git push -d #{branch}",true ,true do|out,err,status|
                            if status!= 0
                                error_branchs.push branch
                            else
                                logN "#{branch}分支删除成功"
                            end
                        end
                    else
                        error_branchs.push branch
                    end
                end
            end
            error_branchs.each do |branch|
                logE "#{branch}分支删除失败"
            end
            logN "总共#{@remote_branchs.length}个分支，计划删除#{@branchs.length}个分支，黑名单跳过#{exclude_count}个，删除失败#{error_branchs.length}个分支"
        end
        def run
            expire_branchs

        end
    end
end
