require 'claide'
require 'json'
# require 'git-thin/utils/utils'

module GitThin
    class ThinConfig
        include GitThinUtils
        DEFAULT = 'default'.freeze
        def initialize(name,keys)
            @name = name
            @keys = keys
            @config = {}
        end

        def name
            return @name
        end
        def keys
            return @keys
        end
        def setValue(value,key = nil )
            if key
                if @keys.include? key
                    @config[key] = value
                end
            else
                for key in @keys
                    @config[key] = value
                end
            end
        end

        def getValue(key = nil )
            if key
                value = @config[key]
                if value.nil? && key == 'path'
                    value = @config['lfs']
                end
                return value
            else
                return @config.values
            end
        end

        def self.prase_config(configs)
            ret_configs = {}
            for config in configs
                items = config.split('=')
                if items.length != 2
                    logE 'error thin config:'+config
                    next
                end
                name = items[0].strip.downcase
                value = items[1].strip.split(',')
                name_items = name.split('.')
                if name_items.length <= 1
                    logE 'error thin config:'+config
                    next
                end
                if name_items[0] != 'thin'
                    # logE 'error thin config:'+config
                    next
                end

                config_name = name_items[1]
                config = ret_configs[config_name]
                if not config
                    config = ThinConfig.new config_name,['lfs','path']
                    ret_configs[config_name] = config
                end

                if name_items.length > 2
                    config_key = name_items[2]
                    config.setValue(value,config_key)
                else
                    config.setValue value
                end
            end

            return ret_configs
        end

        def self.prase_pwd_config(configs,source_root,pwd)
            types = []
            if pwd.index(source_root) == 0
                # 在git 目录中，判断是否命中path路径
                length = source_root.length
                sub = pwd[length+1..-1]
                if sub
                    for key in configs.keys
                        paths = configs[key].getValue('path')
                        for path in paths
                            if sub.index(path) == 0
                                types.push key.downcase
                            end
                        end
                    end
                end
            else
                logW 'You are out of the repo'
            end
            return types
        end

        def self.prase_type_config(argv,configs,source_root=nil ,pwd=nil )
            types = []
            if argv.arguments.length > 0 && argv.arguments[0].index('-')!=0
                first_args = argv.arguments[0].split '|'
                need_shift = false
                for arg in first_args
                    if configs.keys.include? arg.downcase
                        types.push arg.downcase
                        need_shift = true
                    end
                end
                if need_shift
                    argv.shift_argument
                end
            elsif pwd && source_root
                types = types.concat prase_pwd_config(configs,source_root,pwd)
            end
            if types.length == 0
                types.push(DEFAULT)
            end
            return types
        end

    end
    class Config < Thin
        include GitThinUtils
        self.summary = 'Config lfs with .thinconfig'

        self.arguments = [
          CLAide::Argument.new('TYPE', false )
        ]
        self.description = <<-DESC
            According to .thinconfig, configure the platform directory to LFS
        DESC
        def self.options
            [
              ['--source_root', 'Specify the warehouse address manually if necessary.'],
              ['--store', 'Store config to lfs'],
              ['--list', 'List the configs supported by the locally repository, (configured by.thinconfig) '],
              ['--current', 'Displays the current configuration'],
              ['--reset', 'Reset config to thin and lfs'],
              ['--check', 'Check the integrity of the config'],
            ].concat(super)
        end
        def initialize(argv)
            super
            @pwd = Dir.pwd
            @types = []
            @configs = {}
            @source_root = argv.option('source_root')
            @store = argv.flag?('store')
            @list = argv.flag?('list')
            @current = argv.flag?('current')
            @reset = argv.flag?('reset')
            @check = argv.flag?('check')

            if not @source_root
                run_shell 'git rev-parse --show-toplevel',false ,LOGNone do |out,err,status|
                    if status == 0
                        @source_root = out[0].strip
                    end
                end

                if not File.exist? @source_root+'/.git'
                    UI.puts "git repository not found"
                    exit 1
                end
            else
                if @source_root[-1] == '/'
                    @source_root = @source_root[0..-2 ]
                end
            end
            if not Dir.exist? @source_root
                @source_root = nil
                return
            end
            if FileTest.exist? @source_root+'/.thinconfig'
                run_shell "git config -lf #{@source_root}/.thinconfig",true ,true do |out,err,status|
                    @configs = ThinConfig.prase_config out
                end
            end
            @types =ThinConfig.prase_type_config  argv,@configs
        end

        def validate!
            super
            help! 'validate SOURCE_ROOT is required' unless @source_root
        end

        def git_config
            if not @git_configs
                run_shell 'git config -l',true ,true do |outs,errs,status|
                    @git_configs = ThinConfig.prase_config outs
                end
            end
        end
        def run_config
            values=[]
            for type in @types
                config = @configs[type]
                if type == ThinConfig::DEFAULT
                    git_config
                    config = @git_configs[ThinConfig::DEFAULT]
                end
                if config
                    values.push config.getValue('lfs')
                end
            end
            if values.length > 0
                run_shell "git config thin.#{ThinConfig::DEFAULT} "+values.join(','),true ,true
                if @store
                    run_shell "git config lfs.fetchinclude "+values.join(',')
                end
            end
            return values
        end

        def list
            logN 'List the configs supported by the locally repository: '
            git_config
            config = @git_configs[ThinConfig::DEFAULT]
            if config
                log = "name:#{config.name}"
                for key in config.keys
                    log += " #{key}:#{config.getValue key}"
                end
                logN log
            end
            for config in @configs.values
                log = "name:#{config.name}"
                for key in config.keys
                    log += " #{key}:#{config.getValue key}"
                end
                logN log
            end
        end
        def reset
            run_shell "git config --unset thin.#{ThinConfig::DEFAULT} ",true ,true
            run_shell "git config --unset lfs.fetchinclude ",true ,true
        end
        def current

            types = ThinConfig::prase_pwd_config(@configs,@source_root,@pwd)
            if types.length == 0
                types.push ThinConfig::DEFAULT
            end
            for type in types
                config = @configs[type]
                if type == ThinConfig::DEFAULT
                    git_config
                    config = @git_configs[ThinConfig::DEFAULT]
                end
                if config
                    logN "name:#{type}  lfs:#{config.getValue('lfs')}"
                end
            end
        end

        def match_config(match_file)
            for config in @configs.values
                lfs = config.getValue 'lfs'
                if lfs
                    lfs = lfs.map do|file|
                        file.strip
                    end
                    for lf in lfs
                        if match_file.index(lf) == 0
                            return config
                        end
                    end
                end
            end
            return nil
        end
        def check
            lfs_files = []
            run_shell "git lfs ls-files",false ,LOGNone do |outs,errs,status|
                if status
                    for line in outs
                        line[13..-2]
                        lfs_files.push line[13..-2]
                    end
                end
            end
            unmatch_files = []
            for lfs_file in lfs_files

                if match_config(lfs_file).nil?
                    unmatch_files.push lfs_file
                end
            end
            if unmatch_files.length > 0
                logN 'The following LFS files are not configured:'
                for unmatch_file in unmatch_files
                    logW unmatch_file
                end
            else
                logN 'All LFS files are configured'
            end
        end
        def run
            Dir.chdir @source_root
            if @list
                list
            elsif @reset
                reset
            elsif @current
                current
            elsif @check
                check
            else
                values = run_config
                if values.length > 0
                    logN "git config thin.#{ThinConfig::DEFAULT} "+values.join(",")
                else
                    logW "not match analyze config"
                end
            end
            Dir.chdir @pwd
        end
    end
end